@@ -35,6 +35,10 @@
 #include <stdio.h>
 #include <string.h>
 
+#ifdef HAVE_INTTYPES_H
+#include <inttypes.h>
+#endif
+
 #include "extended.h"
 #include "audiofile.h"
 #include "util.h"
@@ -45,19 +49,19 @@
 #include "track.h"
 #include "marker.h"
 
-static status ParseFVER (AFfilehandle file, AF_VirtualFile *fh, u_int32_t type,
+static status ParseFVER (AFfilehandle file, AFvirtualfile *fh, u_int32_t type,
 	size_t size);
-static status ParseAESD (AFfilehandle file, AF_VirtualFile *fh, u_int32_t type,
+static status ParseAESD (AFfilehandle file, AFvirtualfile *fh, u_int32_t type,
 	size_t size);
-static status ParseMiscellaneous (AFfilehandle file, AF_VirtualFile *fh,
+static status ParseMiscellaneous (AFfilehandle file, AFvirtualfile *fh,
 	u_int32_t type, size_t size);
-static status ParseINST (AFfilehandle file, AF_VirtualFile *fh, u_int32_t type,
+static status ParseINST (AFfilehandle file, AFvirtualfile *fh, u_int32_t type,
 	size_t size);
-static status ParseMARK (AFfilehandle file, AF_VirtualFile *fh, u_int32_t type,
+static status ParseMARK (AFfilehandle file, AFvirtualfile *fh, u_int32_t type,
 	size_t size);
-static status ParseCOMM (AFfilehandle file, AF_VirtualFile *fh, u_int32_t type,
+static status ParseCOMM (AFfilehandle file, AFvirtualfile *fh, u_int32_t type,
 	size_t size);
-static status ParseSSND (AFfilehandle file, AF_VirtualFile *fh, u_int32_t type,
+static status ParseSSND (AFfilehandle file, AFvirtualfile *fh, u_int32_t type,
 	size_t size);
 
 _InstParamInfo _af_aiff_inst_params[_AF_AIFF_NUM_INSTPARAMS] =
@@ -97,7 +101,7 @@ _AFfilesetup _af_aiff_default_filesetup =
 /*
 	FVER chunks are only present in AIFF-C files.
 */
-static status ParseFVER (AFfilehandle file, AF_VirtualFile *fh, u_int32_t type, size_t size)
+static status ParseFVER (AFfilehandle file, AFvirtualfile *fh, u_int32_t type, size_t size)
 {
 	u_int32_t	timestamp;
 
@@ -113,7 +117,7 @@ static status ParseFVER (AFfilehandle file, AF_VirtualFile *fh, u_int32_t type,
 /*
 	Parse AES recording data.
 */
-static status ParseAESD (AFfilehandle file, AF_VirtualFile *fh, u_int32_t type, size_t size)
+static status ParseAESD (AFfilehandle file, AFvirtualfile *fh, u_int32_t type, size_t size)
 {
 	_Track		*track;
 	unsigned char	aesChannelStatusData[24];
@@ -141,7 +145,7 @@ static status ParseAESD (AFfilehandle file, AF_VirtualFile *fh, u_int32_t type,
 	Parse miscellaneous data chunks such as name, author, copyright,
 	and annotation chunks.
 */
-static status ParseMiscellaneous (AFfilehandle file, AF_VirtualFile *fh,
+static status ParseMiscellaneous (AFfilehandle file, AFvirtualfile *fh,
 	u_int32_t type, size_t size)
 {
 	int	misctype = AF_MISC_UNRECOGNIZED;
@@ -183,7 +187,7 @@ static status ParseMiscellaneous (AFfilehandle file, AF_VirtualFile *fh,
 	Parse instrument chunks, which contain information about using
 	sound data as a sampled instrument.
 */
-static status ParseINST (AFfilehandle file, AF_VirtualFile *fh, u_int32_t type,
+static status ParseINST (AFfilehandle file, AFvirtualfile *fh, u_int32_t type,
 	size_t size)
 {
 	_Instrument	*instrument;
@@ -271,7 +275,7 @@ static status ParseINST (AFfilehandle file, AF_VirtualFile *fh, u_int32_t type,
 /*
 	Parse marker chunks, which contain the positions and names of loop markers.
 */
-static status ParseMARK (AFfilehandle file, AF_VirtualFile *fh, u_int32_t type,
+static status ParseMARK (AFfilehandle file, AFvirtualfile *fh, u_int32_t type,
 	size_t size)
 {
 	_Track		*track;
@@ -286,7 +290,8 @@ static status ParseMARK (AFfilehandle file, AF_VirtualFile *fh, u_int32_t type,
 	numMarkers = BENDIAN_TO_HOST_INT16(numMarkers);
 
 	track->markerCount = numMarkers;
-	track->markers = _af_marker_new(numMarkers);
+	if (numMarkers)
+		track->markers = _af_marker_new(numMarkers);
 
 	for (i=0; i<numMarkers; i++)
 	{
@@ -335,7 +340,7 @@ static status ParseMARK (AFfilehandle file, AF_VirtualFile *fh, u_int32_t type,
 	sampling rate, the number of sample frames, and the number of
 	sound channels.
 */
-static status ParseCOMM (AFfilehandle file, AF_VirtualFile *fh, u_int32_t type,
+static status ParseCOMM (AFfilehandle file, AFvirtualfile *fh, u_int32_t type,
 	size_t size)
 {
 	_Track		*track;
@@ -358,7 +363,7 @@ static status ParseCOMM (AFfilehandle file, AF_VirtualFile *fh, u_int32_t type,
 	track->f.sampleWidth = BENDIAN_TO_HOST_INT16(sampleSize);
 
 	af_fread(sampleRate, 10, 1, fh);
-	track->f.sampleRate = ConvertFromIeeeExtended(sampleRate);
+	track->f.sampleRate = _af_convert_from_ieee_extended(sampleRate);
 
 	track->f.compressionType = AF_COMPRESSION_NONE;
 	track->f.sampleFormat = AF_SAMPFMT_TWOSCOMP;
@@ -398,6 +403,20 @@ static status ParseCOMM (AFfilehandle file, AF_VirtualFile *fh, u_int32_t type,
 		{
 			track->f.compressionType = AF_COMPRESSION_G711_ALAW;
 		}
+		else if (!memcmp(compressionID, "fl32", 4) ||
+			!memcmp(compressionID, "FL32", 4))
+		{
+			track->f.sampleFormat = AF_SAMPFMT_FLOAT;
+			track->f.sampleWidth = 32;
+			track->f.compressionType = AF_COMPRESSION_NONE;
+		}
+		else if (!memcmp(compressionID, "fl64", 4) ||
+			!memcmp(compressionID, "FL64", 4))
+		{
+			track->f.sampleFormat = AF_SAMPFMT_DOUBLE;
+			track->f.sampleWidth = 64;
+			track->f.compressionType = AF_COMPRESSION_NONE;
+		}
 	}
 
 	_af_set_sample_format(&track->f, track->f.sampleFormat, track->f.sampleWidth);
@@ -409,7 +428,7 @@ static status ParseCOMM (AFfilehandle file, AF_VirtualFile *fh, u_int32_t type,
 	Parse the stored sound chunk, which usually contains little more
 	than the sound data.
 */
-static status ParseSSND (AFfilehandle file, AF_VirtualFile *fh, u_int32_t type,
+static status ParseSSND (AFfilehandle file, AFvirtualfile *fh, u_int32_t type,
 	size_t size)
 {
 	_Track		*track;
